!
!  ATMS 502 / CSE 566 -- Spring, 2019
!  Code for pgm1:  Linear and nonlinear advection
!  =====>>>>> PUT YOUR NAME HERE! and in "name" variable below <<<<<=====
!
  program pgm1

  USE global_data
! Variables from global_data that are used here:
! nx       the number of physical grid points - Not including 'ghost points'
! maxstep  the maximum number of time steps.
! c        [constant] flow speed (e.g. meters/sec) - used only in linear case
! dx       grid spacing: distance (e.g. meters) between grid points in X, Y
! name     a variable containing your name - used to label the plots.
! s1,s2    array data for current and future time
! strue    array with the initial condition, which is also the true solution
! strace   time dimension array storing the max value of s2() at each step
! history  (nx,maxstep) array storing the entire solution for all steps

  implicit none

! Interface statements in Fortran are, like Prototype statements in C, designed
! to take care of the most annoying and (sometimes) difficult problems to debug:
! when the list of arguments passed to a subroutine disagrees with the list
! expected by the subroutine.  Differences in *number* or *type* or arguments
! can cause unexpected results.  Interface statements help avoid these errors.
! You need Interface statements for ALL user subroutines you will be calling.
! Info:  https://pages.mtu.edu/~shene/COURSES/cs201/NOTES/chap06/interface.html

  INTERFACE
     subroutine advection(dt,advection_type)
       real, intent(in)      :: dt
       character, intent(in) :: advection_type
     end subroutine advection
    
     subroutine bc
     end subroutine bc

     subroutine ic
     end subroutine ic

     subroutine plot1d(s,nx,n,smax,overlay,strue,title,name)
       integer, intent(in)             :: nx,n
       real, intent(in)                :: smax
       real, dimension(nx), intent(in) :: s,strue
       logical, intent(in)             :: overlay
       character*(*), intent(in)       :: title,name
     end subroutine plot1d

     subroutine sfc(s,nx,ny,simtime,angh,angv,label,name)
       integer, intent(in)                :: nx,ny
       real, intent(in)                   :: angh,angv,simtime
       real, dimension(nx,ny), intent(in) :: s
       character*(*), intent(in)          :: label,name
     end subroutine sfc

     subroutine stats(s2,nx,n,smax)
       integer, intent(in)                 :: n,nx
       real, intent(out)                   :: smax
       real, dimension(0:nx+1), intent(in) :: s2
     end subroutine stats

     subroutine update
     end subroutine update
  END INTERFACE

! ... Local variables (everything else in the global_data module)
  real         :: dt,courant,smax
  integer      :: i,n,nstep,nplot
  character*1  :: reply,advection_type

! ... Start, print, input ...........................................

  print*,'Program #1    Numerical Fluid Dynamics, Spring 2019'
  print*,' '

  print*,'Enter courant number (normally <= 1; use 1.0 for nonlinear):'
  read*,courant
  dt = courant*dx/c

  write(6,1) courant, dt, nint( dx*real(nx) / c / dt )
1 format(' For courant number ',f5.2,', time step dt = ',f6.3,/, &
         ' Number of time steps for one complete cycle  = ',i3)

  print*,'Enter number of time steps to take:'
  read*,nstep
  if (nstep > maxstep) then
    print*,'nstep too large: exceeds maxstep, which equals',maxstep
    print*,'increase maxstep and recompile.'
    stop
  endif

  print*,'Enter plot interval, in steps (1=every step):'
  read*,nplot

  print*,'Enter L for linear, N for nonlinear advection:'
  read*,advection_type

! ... Open the NCAR Graphics package and set colors.
! ... To invert default black/white colors, comment out calls to gscr below.
! ... Information:  http://ngwww.ucar.edu

  call opngks
  call gscr(1,0,1.,1.,1.)
  call gscr(1,1,0.,0.,0.)

! ... X/Y-axis labels

  call anotat('X','S',0,0,0,0)

! ... Set default Y plot bounds.
 
  call agsetf('Y/MINIMUM.',-1.2)        ! you will need to ...
  call agsetf('Y/MAXIMUM.', 1.2)        ! change these in later codes.

! ... Set and plot the initial condition

  call ic
  call stats(s1,nx,0,smax)
  call plot1d(s1(1:nx),nx,0,smax,.false.,strue(1:nx),'Initial condition',name)

! ... Copy the initial condition (IC) to the "strue" array.
!     We use it later since the initial condition is the true final solution.

  strue(1:nx) = s1(1:nx)

! ... Integrate .....................................................

  do n = 1,nstep

!   Set boundary conditions
    call bc

!   Compute values at next step
    call advection(dt,advection_type)

!   Do array update at end of time step: s2 copied over s1.
    call update

!   Stats
    call stats(s2,nx,n,smax)

!   Copy latest solution s2(1:nx) to history() array,
!    and the s2 maximum value to array strace().
    history(1:nx,n) = s2(1:nx)
    strace(n) = smax

!   Plot fields when needed
    if (n .eq. nstep .or. mod(n,nplot).eq.0) then
      if (advection_type == 'L') then
        call plot1d(s2(1:nx),nx,n,smax,.true. ,strue(1:nx),title,name)
      else
        call plot1d(s2(1:nx),nx,n,smax,.false.,strue(1:nx),title,name)
      endif
    endif

!   Check if problem out of bounds
    if (smax .gt. 1.5) then
      print*,'Stop - solution blowing up at step',n
      call plot1d(s2(1:nx),nx,n,smax,.false.,strue(1:nx),'Solution blowing up',name)
      nstep = n
      exit
    endif

!   end of time loop from n=1 to nstep
  enddo

! ... Run complete - do final plots .................................

! ... Plot Smax(t)

  call agsetf('Y/MINIMUM.',0.0)
  call agsetf('Y/MAXIMUM.',1.5)
  call anotat('N','Max value',0,0,0,0)
  call stats(strace,nx,nstep,smax)
  call plot1d(strace,nstep,0,smax,.false.,strue(1:nx),'Smax vs. time',name)

! ... Plot history (time-space) surface s(x,t)
! ... Three different viewpoints are used

  call sfc(history,nx,nstep,dt*real(nstep),-90., 0.,    &
      	   'Time-Space evolution (view: -90,  0)',name)
  call sfc(history,nx,nstep,dt*real(nstep),-75., 5.,    &
      	   'Time-Space evolution (view: -75, +5)',name)
  call sfc(history,nx,nstep,dt*real(nstep),-30.,20.,    &
      	   'Time-Space evolution (view: -30,+20)',name)

! ... Close graphics package and stop.

  call clsgks
  stop
  end program pgm1
