! ========================= plot1d ====================
! Plot1d plots a one-dimensional function using NCAR graphics
! If "overlay" is true, we overlay the "true" solution as dashed line.
!
! Arguments (all are input):
!
!   s	     real array   array to plot (important: only pass the
!                           portion of array to plot - no ghost points!)
!   nx	     integer      size of array, not including ghost points
!   n	     integer      time step counter, from main program
!   smax     real         max absolute value in s array - used in label
!   overlay  logical      if true, plot will be an overlay of 2 fields
!   strue    real array   [optional] overlay field, ignored if overlay=false
!   title    char*(*)     title (time step, max info appended to this)
!   name     char*(*)     name to place on plot
!
    subroutine plot1d(s,nx,n,smax,overlay,strue,title,name)
    implicit none

    integer, intent(in)               :: nx,n
    real, intent(in)                  :: smax
    real, dimension(nx), intent(in)   :: s,strue
    logical, intent(in)               :: overlay
    character*(*), intent(in)         :: title,name

    integer                           :: i
    real, allocatable, dimension(:,:) :: both
    character*100                     :: label
    character*16                      :: AGDSHN

    write(label,1) title,n,smax
1   format(a,'  N=',i4,' Max=',f8.5)

    write(6,2) label(1:(21+len(title)))
2   format('Plotting ',a)

!   If overlay is false, simply plot s()
!   If overlay is true , fill both() arrays with both fields, and plot that

!   suppress automatic graphics frame advance for now
    call agseti('frame.',2)

    if (.not.overlay) then

      call ezy(s,nx,label)

    else

      allocate(both(nx,2))
      do i = 1,nx
        both(i,1) = s(i)
        both(i,2) = strue(i)
      enddo
      call agsetr('DASH/SELECTOR.',2.)
      call agseti('DASH/LENGTH.'  ,20)
      call agsetc(AGDSHN(1),'$$$$$$$$$$$$$$$$$$$$')
      call agsetc(AGDSHN(2),'$$''''$$''''$$''''$$''''$$''''')
      call ezmy(both,nx,2,nx,label)
      deallocate(both)
    endif

! ... add additional labels

    call set(.05,.95,.05,.95,0.,1.,0.,1.,1)
    call pcmequ(0.02,1.0,name,0.01,90.,1.)
    call pcmequ(1.0,0.05,'ATMS 502/CSE 566',0.01,0.,1.)
    call pcmequ(0.0,0.05,'Spring 2019',0.01,0.,-1.)

! ... done

    call frame
!   restore automatic graphics frame advance
    call agseti('frame.',1)
    return

    end subroutine plot1d
