! =========================== sfc =====================
!  sfc plots a 2-d field as a surface and shows the max/min, time & label.
!  NCAR Graphics routine ezsrfc plots the surface.
!  Arguments:
!
!   s       input  real array    field to be displayed.
!   nx,ny   input  integers      dimensions of 's'
!   simtime input  real          time of integration
!   angh    input  real          horizontal viewing angle counter-
!                                  clockwise from x-axis
!   angv    input  real          vertical viewing angle; >0=above plane
!                                  of field average.
!   label  input   character*(*) character label
!   name   input   character*(*) your name, to place on plot
!
    subroutine sfc(s,nx,ny,simtime,angh,angv,label,name)
    implicit none

    integer, intent(in)                :: nx,ny
    real, intent(in)                   :: angh,angv,simtime
    real, dimension(nx,ny), intent(in) :: s
    character*(*), intent(in)          :: label,name

    integer                            :: i,j,mmlen
    real                               :: smin,smax
    real, allocatable, dimension(:)    :: workarray
    character*15                       :: tlabel
    character*45                       :: mmlabel

! ... Find min and max of arrays to be plotted and set up labels

    smax = maxval(s)
    smin = minval(s)
!   write(*,*) 'Max Value = ',smax,' Min value = ',smin

! ... Write labels into character strings

    write(tlabel,1) simtime
1   format('TIME =',f7.4)

    if (abs(smin).ge.100.0.or. abs(smax).ge.100.0) then
      write(mmlabel,2) smin,smax
2     format('MIN = ',e13.4,'  MAX = ',e13.4)
      mmlen = 40

    else
      write(mmlabel,3) smin,smax
3     format('MIN=',f7.3,'  MAX=',f6.3)
      mmlen = 23
    endif

! ... Plot labels

    call set(0.,1.,0.,1.,0.,1.,0.,1.,1)
    call pcmequ(0.50,0.97,  label         ,0.020,0.0, 0.0)
    call pcmequ(0.95,0.02,mmlabel(1:mmlen),0.015,0.0, 1.0)
    call pcmequ(0.05,0.02, tlabel         ,0.015,0.0,-1.0)

! ... Additional labels

    call pcmequ(0.02,0.99,name,0.01,90.,1.)
    call pcmequ(0.98,0.06,'ATMS 502/CSE 566',0.01,0.,1.)
    call pcmequ(0.02,0.06,'Spring 2019',0.01,0.,-1.)

! ... Allocate workarray for use by graphics routine

    allocate(workarray( (2*nx*ny)+nx+ny) )

! ... Call EZSRFC to show the data array
 
    call ezsrfc(s,nx,ny,angh,angv,workarray)

    deallocate(workarray)
    return

    end subroutine sfc
